#!/usr/bin/env python3


#% $Id$ 

#
# Copyright (C) 2002-2007
# ASTRON (Netherlands Foundation for Research in Astronomy)
# and The MeqTree Foundation
# P.O.Box 2, 7990 AA Dwingeloo, The Netherlands, seg@astron.nl
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#
#  (c) 2013.				 (c) 2011.
#  National Research Council		 Conseil national de recherches
#  Ottawa, Canada, K1A 0R6 		 Ottawa, Canada, K1A 0R6
#
#  This software is free software;	 Ce logiciel est libre, vous
#  you can redistribute it and/or	 pouvez le redistribuer et/ou le
#  modify it under the terms of	         modifier selon les termes de la
#  the GNU General Public License	 Licence Publique Generale GNU
#  as published by the Free		 publiee par la Free Software
#  Software Foundation; either	 	 Foundation (version 3 ou bien
#  version 2 of the License, or	 	 toute autre version ulterieure
#  (at your option) any later	 	 choisie par vous).
#  version.
#
#  This software is distributed in	 Ce logiciel est distribue car
#  the hope that it will be		 potentiellement utile, mais
#  useful, but WITHOUT ANY		 SANS AUCUNE GARANTIE, ni
#  WARRANTY; without even the	 	 explicite ni implicite, y
#  implied warranty of			 compris les garanties de
#  MERCHANTABILITY or FITNESS FOR	 commercialisation ou
#  A PARTICULAR PURPOSE.  See the	 d'adaptation dans un but
#  GNU General Public License for	 specifique. Reportez-vous a la
#  more details.			 Licence Publique Generale GNU
#  					 pour plus de details.
#
#  You should have received a copy	 Vous devez avoir recu une copie
#  of the GNU General Public		 de la Licence Publique Generale
#  License along with this		 GNU en meme temps que ce
#  software; if not, contact the	 logiciel ; si ce n'est pas le
#  Free Software Foundation, Inc.	 cas, communiquez avec la Free
#  at http://www.fsf.org.		 Software Foundation, Inc. au
#						 http://www.fsf.org.
#
#  email:				 courriel:
#  business@hia-iha.nrc-cnrc.gc.ca	 business@hia-iha.nrc-cnrc.gc.ca
#
#  National Research Council		 Conseil national de recherches
#      Canada				    Canada
#  Herzberg Institute of Astrophysics	 Institut Herzberg d'astrophysique
#  5071 West Saanich Rd.		 5071 West Saanich Rd.
#  Victoria BC V9E 2E7			 Victoria BC V9E 2E7
#  CANADA					 CANADA
#
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division

import sys
import numpy
import math

# Maps a double interval into an integer interval

# The ImageScaler class maps an interval of type double into an interval of
# type int. It consists of two intervals D = [d1, d2] (double) and 
# I = [i1, i2] (int), which are specified with the setDblRange and setIntRange
# members. The point d1 is mapped to the point i1, and d2 is mapped to i2. 
# Any point inside or outside D can be mapped to a point inside or outside
# I using transform or limTransform or vice versa using invTransform. 
# D can be scaled linearly or logarithmically, as specified with setDblRange.
#
# This is a translation of the QWT Package's QwtDiMap class to work 
# directly with python numpy arrays

class ImageScaler:
  LogMin = 1.0e-150
  LogMax = 1.0e150

  def __init__(self, i1=0, i2=1, d1=0.0, d2=1.0, logarithmic=False):
    self.d_x1 = float(d1)        # lower bound of double interval
    self.d_x2 = float(d2)        # upper bound of double interval
    self.d_y1 = i1               # lower bound of integer interval
    self.d_y2 = i2               # upper bound of integer interval
    self.d_cnv = 1.0             # conversion factor
    self.d_log = logarithmic     # logarithmic scale?
    self.setDblRange(self.d_x1, self.d_x2, logarithmic)

  def d1(self):
# return the first border of the double interval
    return self.d_x1

  def d2(self):
# return the second border of the double interval
    return self.d_x2

  def i1(self):
# return the first border of the integer interval
    return self.d_y1

  def i2(self):
# return the second border of the integer interval
    return self.d_y2

  def logarithmic(self):
# return TRUE if the double interval is scaled logarithmically
    return self.d_log

  def cnv(self):
# return the conversion factor
    return self.d_cnv

  def iTransform(self,x):
#  Transform a point in double interval into an point in the
#  integer interval
#  param: x floating point array to be transformed
#  linear mapping:<dd>rint(i1 + (i2 - i1) / (d2 - d1) * (x - d1))
#  logarithmic mapping:<dd>rint(i1 + (i2 - i1) / log(d2 / d1) * log(x / d1))
#  warning: The specified point is allowed to lie outside the intervals. If you
#  want to limit the returned value, use the limTransform method.
    temp_array = None
    if self.d_log:
        temp_array = numpy.around((numpy.log(x) - self.d_x1) * self.d_cnv)
    else:
        temp_array = numpy.around((x - self.d_x1) * self.d_cnv)
    return self.d_y1 + temp_array.astype(numpy.int32)


  def setDblRange(self, d1, d2, lg=False):
# Specify the borders of the double interval
# param: d1 first border
# param: d2 second border 
# param: lg logarithmic (TRUE) or linear (FALSE) scaling
    if lg:
        self.d_log = True;
        if d1 < self.LogMin: 
           d1 = self.LogMin
        elif d1 > self.LogMax: 
           d1 = self.LogMax
        
        if d2 < self.LogMin: 
           d2 = self.LogMin
        elif d2 > self.LogMax: 
           d2 = self.LogMax
        
        self.d_x1 = math.log(d1);
        self.d_x2 = math.log(d2);
    else:
        self.d_log = False
        self.d_x1 = d1
        self.d_x2 = d2
    self.newFactor();

  def newFactor(self):
# Re-calculate the conversion factor.
    if self.d_x2 != self.d_x1:
       self.d_cnv = 1.0 * (self.d_y2 - self.d_y1) / (self.d_x2 - self.d_x1); 
    else: 
       d_cnv = 0.0;

  def setIntRange(self, i1, i2):
# Specify the borders of the integer interval
# param: i1 first border
# param: i2 second border
    self.d_y1 = i1
    self.d_y2 = i2
    self.newFactor()

  def invTransform(self, y):
# Transform an integer value into a double value
# param: y integer array to be transformed
# linear mapping:<dd>d1 + (d2 - d1) / (i2 - i1) * (y - i1)
# logarithmic mapping:<dd>d1 + (d2 - d1) / log(i2 / i1) * log(y / i1)
    if self.d_cnv == 0.0:
       return 0.0;
    else:
       temp_array = self.d_x1 + ((y - self.d_y1) / self.d_cnv).astype(numpy.float64) 
       if self.d_log: 
           return numpy.exp(temp_array)
       else:
           return temp_array

  def limTransform(self, x):
# Transform and limit
# The function is similar to transform, but limits the input value
# to the nearest border of the map's double interval if it lies outside
# that interval.
# param: x array to be transformed
# return transformed value
    if self.d_log: 
      if x > self.LogMax:
          x = self.LogMax
      elif x < self.LogMin:
          x = self.LogMin
      x = numpy.log(x)
    
    if  x > max(self.d_x1, self.d_x2):
      x = max(self.d_x1, self.d_x2);
    elif  x < min(self.d_x1, self.d_x2):
      x = min(self.d_x1, self.d_x2);

    if self.d_log:
      return self.transform(numpy.exp(x))
    else:
      return self.transform(x)

  def xTransform(self, x):
# Exact transformation
# This function is similar to transform, but
# makes the integer interval appear to be double. 
# param: x array to be transformed
# linear mapping:<dd>i1 + (i2 - i1) / (d2 - d1) * (x - d1)
# logarithmic mapping:<dd>i1 + (i2 - i1) / log(d2 / d1) * log(x / d1)
    if self.d_log:
       rv = float(self.d_y1) + (numpy.log(x) - self.d_x1) * self.d_cnv;    
    else:
       rv = float(self.d_y1) + (x - self.d_x1) * self.d_cnv;
    return rv;


  def transform(self, x):
#  Transform a point in double interval into an point in the
#  integer interval
# param x value
# return:
# linear mapping:<dd>rint(i1 + (i2 - i1) / (d2 - d1) * (x - d1))
# logarithmic mapping:<dd>rint(i1 + (i2 - i1) / log(d2 / d1) * log(x / d1))

# warning The specified point is allowed to lie outside the intervals. If you
# want to limit the returned value, use QwtDiMap::limTransform.
    if self.d_log:
        rv =  self.d_y1 + numpy.ceil((numpy.log(x) - self.d_x1) * self.d_cnv)
    else:
        rv = self.d_y1 + numpy.ceil((x - self.d_x1) * self.d_cnv)
    return rv

def main(args):
  b = 1 + numpy.array(list(range(1000)))
  c = 1.0 * b
  a = ImageScaler(1, 256, c.min(), c.max(), True)
  print(a.logarithmic())
  print(a.d2())
  print(a.cnv())
  d = a.iTransform(c)
  print('d = ',  d)
  d = a.xTransform(c)
  print('d = ',  d)
  e = a.invTransform(d)
  print('e = ',  e)

if __name__ == '__main__':
    main(sys.argv)




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'buffersizedialog.ui'
#
# Created: Wed Jan 25 11:40:15 2006
#      by: The PyQt User Interface Compiler (pyuic) 3.13
#
# WARNING! All changes made in this file will be lost!


#% $Id: BufferSizeDialog.py 5418 2007-07-19 16:49:13Z oms $ 

#
# Copyright (C) 2002-2007
# ASTRON (Netherlands Foundation for Research in Astronomy)
# and The MeqTree Foundation
# P.O.Box 2, 7990 AA Dwingeloo, The Netherlands, seg@astron.nl
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division

import sys

from qwt.qt.QtGui import (QApplication, QDialog, QHBoxLayout,
         QLabel, QSizePolicy, QSlider, QPushButton, QVBoxLayout, QSpinBox, QSpacerItem)
from qwt.qt.QtCore import Qt, QSize, QObject, pyqtSignal
from qwt.qt.QtGui import QWidget, QFont, QFontInfo


class BufferSizeDialog(QDialog):

    return_value = pyqtSignal(int)

    def __init__(self, buffersize = 0, parent = None,name = None,modal = True,fl = 0):
        QDialog.__init__(self,parent)

        self.setModal(modal)

        BufferSizeDialogLayout = QVBoxLayout(self)

        layout1 = QHBoxLayout(None)

        self.label = QLabel(self)
        layout1.addWidget(self.label)

        self.spinBox1 = QSpinBox(self)
        self.spinBox1.setMinimum(0)
        self.spinBox1.setWrapping(True)
        self.spinBox1.setValue(buffersize)
        self.value = buffersize

        layout1.addWidget(self.spinBox1)
        BufferSizeDialogLayout.addLayout(layout1)

        layout3 = QHBoxLayout(None)
        spacer1 = QSpacerItem(71,31,QSizePolicy.Expanding,QSizePolicy.Minimum)
        layout3.addItem(spacer1)

        self.okButton = QPushButton("okButton",self)
        layout3.addWidget(self.okButton)

        self.cancelButton = QPushButton("cancelButton",self)
        layout3.addWidget(self.cancelButton)
        BufferSizeDialogLayout.addLayout(layout3)

        self.resize(QSize(267,84).expandedTo(self.minimumSizeHint()))

        self.okButton.clicked.connect(self.runDone)
        self.cancelButton.clicked.connect(self.runCancel)
        self.spinBox1.valueChanged.connect(self.updateValue)

        self.label.setBuddy(self.spinBox1)



    def updateValue(self,a0):
        self.value = a0

    def runDone(self):
        self.return_value.emit(self.value)
        self.done(self.value)

    def runCancel(self):
        self.value = -1
        self.runDone()

def main(args):
    app = QApplication(args)
    demo = BufferSizeDialog(10)
    demo.show()
    app.exec_()

# main()

# Admire
if __name__ == '__main__':
    main(sys.argv)




#!/usr/bin/env python3

#
#% $Id$ 
#
#
# Copyright (C) 2002-2007
# The MeqTree Foundation & 
# ASTRON (Netherlands Foundation for Research in Astronomy)
# P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>,
# or write to the Free Software Foundation, Inc., 
# 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

from PyQt4.Qt import *
from Kittens.widgets import PYSIGNAL

class ProcStatusWidget (QLabel):
  
  def __init__(self,parent,width=None,height=None):
    QLabel.__init__(self,parent);
    # self.setSizePolicy(QSizePolicy(QSizePolicy.Minimum,QSizePolicy.Preferred));
    if width is not None:
      self.setMinimumWidth(width);
    if height is not None:
      self.setMinimumHeight(height);
    self.setIndent(5);
    self.setAlignment(Qt.AlignRight|Qt.AlignVCenter);
    # init status
    self._maxsizes = [0,0,0];
    
  def reset (self):
    self._maxsizes = [0,0,0];
    
  def formatTime(self,sec,usec):
    h = sec/3600;
    sec %= 3600;
    m = sec/60;
    sec %= 60;
    sec += 0.01*(usec/10000);
    if h and m:
      return "<b>%d:%02d:%.2f</b>"%(h,m,sec);
    elif m:
      return "<b>%d:%.2f</b>"%(m,sec);
    else:
      return "<b>%.2f</b>"%(sec,);
  
  def setStatus (self,stat):
    (vsz,rss,shm,code,lib,data,dirty,cpu_sec,cpu_usec,cpus_sec,cpus_usec) = self._stat = stat;
    lbls = ("VSZ","RSS");
    s = "<nobr><small>";
    # form memory size strings
    for (i,val) in enumerate((vsz,rss)):
      val /= 1024;
      self._maxsizes[i] = max(val,self._maxsizes[i]);
      if self._maxsizes[i] != val:
        s += "%s:<b>%d</b>/%dM " % (lbls[i],val,self._maxsizes[i]);
      else:
        s += "%s:<b>%d</b>M " % (lbls[i],val);
    # form cpu time string
    ## disabled for now, since time accounting on 2.6 kernels is broken
    # s += "cpu:" + self.formatTime(cpu_sec,cpu_usec);
    # s += "/" + self.formatTime(cpus_sec,cpus_usec);
    
    s += "</small></nobr>";
    # set label
    self.setText(s);



# -*- coding: utf-8 -*-
from PyQt4.Qt import *
from Kittens.widgets import PYSIGNAL

import Timba
from MeqGUI import Grid, GUI, Plugins
from Timba.dmi import *
from Timba.utils import *
from MeqGUI.GUI.pixmaps import pixmaps
from Timba import TDL

import sys
import os
import os.path

_dbg = verbosity(0,name='tdlerrors');
_dprint = _dbg.dprint;
_dprintf = _dbg.dprintf;

class TDLErrorFloat (QMainWindow,PersistentCurrier):
  """implements a floating window for TDL error reports""";
  def __init__ (self,parent):
    QMainWindow.__init__(self,parent,Qt.Dialog|Qt.WindowTitleHint);
    self.hide();
    # self.setWindowIcon(pixmaps.red_round_cross.icon());
    self.setWindowTitle("TDL Errors");
    # make widgets
    self._werrlist_box = QWidget(self);
    lo = QVBoxLayout(self._werrlist_box);
    lo.setContentsMargins(0,0,0,0);
    lo.setSpacing(0);
    self.setCentralWidget(self._werrlist_box);
    # error list header is a toolbar
    errlist_hdr = QToolBar("TDL errors",self._werrlist_box);
    errlist_hdr.setSizePolicy(QSizePolicy.Preferred,QSizePolicy.Fixed);
    errlist_hdr.setIconSize(QSize(16,16));
    lo.addWidget(errlist_hdr);
    # prev/next error buttons
    self._qa_prev_err = errlist_hdr.addAction(pixmaps.red_leftarrow.icon(),"Show &previous error")
    self._qa_prev_err.setShortcut(Qt.ALT+Qt.Key_P);
    QObject.connect(self._qa_prev_err,SIGNAL("triggered()"),self._show_prev_error);
    self._qa_next_err = errlist_hdr.addAction(pixmaps.red_rightarrow.icon(),"Show &next error");
    self._qa_next_err.setShortcut(Qt.ALT+Qt.Key_N);
    QObject.connect(self._qa_next_err,SIGNAL("triggered()"),self._show_next_error);
    # label with error count
    self._error_count_label = QLabel(errlist_hdr);
    errlist_hdr.addWidget(self._error_count_label);
    self._error_count_label.setSizePolicy(QSizePolicy.Expanding,QSizePolicy.Fixed);
    # error list itself
    self._werrlist = QTreeWidget(self._werrlist_box);
    lo.addWidget(self._werrlist);
    self._werrlist.setSizePolicy(QSizePolicy.MinimumExpanding,QSizePolicy.Preferred);
    QObject.connect(self._werrlist,SIGNAL("currentItemChanged(QTreeWidgetItem*,QTreeWidgetItem*)"),self.curry(self._process_item_click,"currentChanged"));
    QObject.connect(self._werrlist,SIGNAL("itemClicked(QTreeWidgetItem*,int)"),self.curry(self._process_item_click,"clicked"));
    QObject.connect(self._werrlist,SIGNAL("itemActivated(QTreeWidgetItem*,int)"),self.curry(self._process_item_click,"spacePressed"));
    QObject.connect(self._werrlist,SIGNAL("itemExpanded(QTreeWidgetItem*)"),self.curry(self._process_item_click,"expanded"));
    self._werrlist.setColumnCount(4);
    self._werrlist.setSortingEnabled(False);
    self._werrlist.setRootIsDecorated(True);
    try: # only in qt-4.2 and later
      self._werrlist.setAllColumnsShowFocus(True);
    except AttributeError:
      pass;
    header = self._werrlist.header();
    header.hide(); # setHeaderHidden(True);
    header.setStretchLastSection(False);
    for col in [0,2,3]:
      header.setResizeMode(col,QHeaderView.ResizeToContents);
    header.setResizeMode(1,QHeaderView.Stretch);
    self._werrlist.setSelectionMode(QAbstractItemView.SingleSelection);
    # size the widget
    # self.setMinimumSize(QSize(500,120));
    # self.setGeometry(0,0,800,120);
    # anchor and position relative to anchor
    self._anchor_widget = None;
    self._anchor_xy0 = None;
    self._anchor_xy  = 0,0;
    self._current_xy = 0,0;
    self._anchor_ref = 0,0;
    self._anchoring = False;
    # explicit geometry set via anchor, None of none set
    self._geom = None;
    # timer used for move operations
    self._move_timer = QTimer(self);
    self._move_timer.setSingleShot(True);
    # internal state
    self._item_shown = None;
    self._in_show_error_item = False;
    self._error_list = [];

  def closeEvent (self,ev):
    """Window closed: hide and set a flag""";
    self.hide();
    ev.ignore();

  def show (self):
    """Only show the window if we have errors in it""";
    if self._error_list:
      QMainWindow.show(self);

  def moveEvent (self,ev):
    QMainWindow.moveEvent(self,ev);
#    print "moveEvent spontaneous:",ev.spontaneous();
    # ignore move events for some time after a move_anchor() -- these are probably caused
    # by the anchor moving us, and not by the user
    if not self._move_timer.isActive():
      self._current_xy = self.geometry().x(),self.geometry().y();
#      print "current xy is",self._current_xy;

  def showEvent (self,ev):
#    print "showEvent spontaneous:",ev.spontaneous();
    if not ev.spontaneous():
      x,y = self._current_xy;
      self.setGeometry(x,y,self.width(),self.height());
      self._move_timer.start(200);
    QMainWindow.showEvent(self,ev);
#    print "showEvent ends";

  def set_anchor (self,widget,x,y,xref=0,yref=0):
    """Tells the window to anchor itself to point x,y of the given widget.
    If xref is 0, x is relative to the left side, otherwise to the right side.
    If yref is 0, y is relative to the top edge, otherwise to the bottom edge.
    """;
    self._anchor_widget = widget;
    self._anchor_xy = x,y;
    self._anchor_ref = xref,yref;
#    print "anchoring to",x,y,xref,yref;
    self.move_anchor();

  def move_anchor (self):
    """Notifies the window that its anchor widget has moved around.
    Changes position following the anchor.""";
    # get dx,dy coordinates relative to old anchor point
    if self._anchor_xy0 is not None:
      x0,y0 = self._anchor_xy0;
      x,y = self._current_xy;
#      print "move_anchor: old location is",x,y;
      dx = x - x0;
      dy = y - y0;
    else:
      dx = dy = 0;
#    print "move_anchor: dxy relative to old anchor is",dx,dy;
    # compute new anchoring point
    top = self._anchor_widget.mapToGlobal(QPoint(0,0));
    btm = self._anchor_widget.mapToGlobal(QPoint(self._anchor_widget.width(),
                          self._anchor_widget.height()));
    #print "move_anchor: widget top is",top.x(),top.y();
    #print "move_anchor: widget btm is",btm.x(),btm.y();
    x,y = self._anchor_xy;
    # add to coordinates of reference point on anchor
    if self._anchor_ref[0]:
      x0 = btm.x() + x;
    else:
      x0 = top.x() + x;
    if self._anchor_ref[1]:
      y0 = btm.y() + y;
    else:
      y0 = top.y() + y;
    # move to new location
    #print "move_anchor: new location is",x0,y0;
    self._anchor_xy0 = x0,y0;
    self._current_xy = x0+dx,y0+dy;
    self.setGeometry(x0+dx,y0+dy,self.width(),self.height());
    # Start a timer to ignore move events for a bit. The reason for this is that sometimes
    # very rapid moving of the main window causes the float to "lag" because the setGeometry()
    # call is not processed before another call to move_anchor(). A small delay should eliminate this.
    self._move_timer.setSingleShot(True);
    self._move_timer.start(200);

  def _populate_error_list (self,parent,errlist,toplevel=False):
    """helper function to recursively populate the error ListView""";
    previtem = None;
    for index,err in enumerate(errlist):
      if isinstance(err,TDL.TDLError):
        errmsg = str(err.args[0]);
      else:
        errmsg = str(err);
      filename = getattr(err,'filename',None);
      line = getattr(err,'lineno',0);
      column = getattr(err,'offset',0) or 0;  # offset is sometimes None, make sure this is 0
      suberror = isinstance(err,TDL.NestedTDLError);
      _dprint(1,errmsg,"at",filename,line,column);
      # index of corresponding top-level item
      toplevel_index = len(self._toplevel_error_items)-1;
      if toplevel:
        toplevel_index += 1;
      # create item
      if previtem is not None:
        previtem = item = QTreeWidgetItem(parent,previtem);
      else:
        previtem = item = QTreeWidgetItem(parent);
      item.setText(0,"%d:"%(toplevel_index+1,));
      item.setTextAlignment(0,Qt.AlignLeft);
      # add housekeeping info
      item._toplevel_index = toplevel_index;
      if toplevel:
	      self._toplevel_error_items.append(item);
      toplevel_index = len(self._toplevel_error_items)-1;
      self._error_items.append(item);
      item.setExpanded(False);
      # set item content
      if filename is None:
        item.setText(1,errmsg);
        item.setText(2,"<unknown>");
        item.setText(3,"(%s)"%err.__class__.__name__);
      else:
        # normalize filenames: eliminate CWD, leave just one path element
        fname = filename;
        try: is_cwd = os.path.samefile(os.path.dirname(fname),'.');
        except: is_cwd = False;
        if is_cwd:
          fname = os.path.basename(filename);
        else:
          dirname,fname = os.path.split(filename);
          dir1,dir2 = os.path.split(dirname);
          if dir1 == '/' or not dir1:
            fname = filename;
          else:
            fname = os.path.join("...",dir2,fname);
        item.setText(1,errmsg);
        item.setText(3,"(%s)"%err.__class__.__name__);
        item._err_location = len(self._error_items)-1,filename,line,column,suberror;
        self._error_locations.append(item._err_location);
        item.setText(2,"[%s:%d]" % (fname,line));
      # recursively populate with nested errors if needed
      nested = getattr(err,'nested_errors',None);
      if nested:
        self._populate_error_list(item,nested);
    # resize ourselves according to number of errors and width of treeview
    height = (len(errlist)+1)*self._werrlist.fontMetrics().lineSpacing();
    height = min(200,height);
    # work out recommended width
    width = 400;  # initial width for message section
    # add width of other sections
    for col in [0,2,3]:
      width += self._werrlist.header().sectionSize(col);
    _dprint(2,"hinted width is",width);
    self.setGeometry(self.x(),self.y(),max(width,self.width()),height);
    self.updateGeometry();

  def set_errors (self,error_list,emit_signal=True,show_item=True,
                      message="TDL compile failed"):
    """Shows an error list. errlist should be a sequence of Exception
    objects following the TDL error convention.
    message is a status message
    If show_item=True, highlights the first error
    If emit_signal=True, emits a hasErrors(nerr) pysignal
    """;
    _dprint(1,"set_errors: list of",len(error_list),"entries");
    _dprint(1,"current list has",len(self._error_list),"entries");
    # do nothing if already set
    if self._error_list is error_list:
      return;
    self.clear_errors(emit_signal=False);
    self._error_list = error_list;
    self._error_locations = [];         # list of error locations
    self._error_items = [];
    self._toplevel_error_items = [];
    if error_list:
      self._populate_error_list(self._werrlist,error_list,toplevel=True);
      nerr = len(self._toplevel_error_items);
      self._error_count_label.setText('%s: <b>%d</b> errors'%(message,nerr));
      self.setWindowTitle("TDL Errors: %d"%nerr);
      if emit_signal:
        self.emit(PYSIGNAL("hasErrors"),nerr);
      if show_item:
        self._show_error_item(self._toplevel_error_items[0]);
      self.show();
      # self._highlight_error(0);
      # disable run control until something gets modified
      # self._qa_run.setVisible(False);
    else:
      if emit_signal:
        self.emit(PYSIGNAL("hasErrors"),0);
      self.setWindowTitle("TDL Errors");
      self.hide();

  def get_error_list (self):
    return self._error_list;

  def get_error_locations (self):
    return self._error_locations;

  def clear_errors (self,emit_signal=True):
    """clears the error list. If emit_signal=True, emits a hasErrors(0) pysignal""";
    if emit_signal:
      self.emit(PYSIGNAL("hasErrors"),0);
    self._error_items = self._toplevel_error_items = None;
    self.setWindowTitle("TDL Errors");
    self._werrlist.clear();
    self._error_count_label.setText('');
    self._error_list = [];
    self._error_locations = [];
    self._item_shown = None;
    self.hide();

  def _highlight_error_item (self,item,emit_signal=True):
    """highlights the given error item. If emit_signal=True, emits a
    showError(index,filename,line,column) or showError(None) pysignal.
    """;
    if item:
      self.show();
      toplevel_index = item._toplevel_index;
      self._qa_prev_err.setEnabled(toplevel_index > 0);
      self._qa_next_err.setEnabled(toplevel_index < len(self._toplevel_error_items)-1);
      # does item contain a location attribute?
      errloc = getattr(item,'_err_location',None);
      if errloc:
        # indicate location
        _dprint(1,"highlighting error in",errloc);
        if emit_signal:
          self.emit(PYSIGNAL("showError"),*errloc);
        return
    # if we fell through to here, then no error has been shown -- emit appropriate signal
    if emit_signal:
      self.emit(PYSIGNAL("showError"),None,None,None,None);

  def _show_next_error (self):
    item = self._werrlist.currentItem();
    if item:
      toplevel_index = item._toplevel_index + 1;
      if toplevel_index >= len(self._toplevel_error_items):
        toplevel_index = 0;
      item = self._toplevel_error_items[toplevel_index];
    else:
      item = self._error_items[0];
    if item:
      self._show_error_item(item);

  def _show_prev_error (self):
    item = self._werrlist.currentItem();
    if item:
      toplevel_index = item._toplevel_index - 1;
      if toplevel_index < 0:
        toplevel_index = len(self._toplevel_error_items)-1;
      item = self._toplevel_error_items[toplevel_index];
    else:
      item = self._toplevel_error_items[-1];
    if item:
      self._show_error_item(item);

  def show_error_number (self,index,emit_signal=True):
    self._show_error_item(self._error_items[index],emit_signal=emit_signal);

  def _show_error_item (self,item,emit_signal=True):
    # do nothing if item is None, or already shown, or if already within this function
    if item is None or self._in_show_error_item or item is self._item_shown:
      return;
    self._in_show_error_item = True;
    try:
      self._item_shown = item;
      self._werrlist.setCurrentItem(item);
      for other in self._toplevel_error_items:
        if other is item:
          other.setExpanded(True);
        else:
          other.setExpanded(False);
      self._werrlist.scrollToItem(item);
      self._highlight_error_item(item,emit_signal=emit_signal);
    finally:
      self._in_show_error_item = False;

  def _process_item_click (self,why,item,dum=None):
    self._show_error_item(item,emit_signal=True);





#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#
#% $Id$
#
#
# Copyright (C) 2002-2007
# The MeqTree Foundation &
# ASTRON (Netherlands Foundation for Research in Astronomy)
# P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>,
# or write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

import Timba
import MeqGUI
from MeqGUI import Grid, GUI, Plugins
from Timba.dmi import *
from Timba.utils import *
from MeqGUI.GUI.pixmaps import pixmaps
from MeqGUI.Grid.Debug import *
from Timba import *
from MeqGUI import GUI, Plugins, Grid

import weakref
import re
import gc
import types

from PyQt4.Qt import *
from Kittens.widgets import PYSIGNAL

# ====== Grid.Workspace ======================================================
# implements a multi-page, multi-panel viewing grid
#
class Workspace (object):
  # define a toolbutton that accepts drops of DataItems
  class DataDropButton(GUI.widgets.DataDroppableWidget(QToolButton)):
    def accept_drop_item_type (self,itemtype):
      return issubclass(itemtype,Grid.DataItem);

  def __init__ (self,parent,max_nx=4,max_ny=4,use_hide=None):
    # dictionary of UDIs -> list of GridDataItem objects
    self._dataitems = dict();
    # highlighted item
    self._highlight = None;
    # currier
    self._currier = PersistentCurrier();
    self.curry = self._currier.curry;
    self.xcurry = self._currier.xcurry;

    self._maintab = QTabWidget(parent);
    self._maintab.setTabPosition(QTabWidget.North);
    QWidget.connect(self._maintab,SIGNAL("currentChanged(int)"),self._set_layout_button);
    self.max_nx = max_nx;
    self.max_ny = max_ny;
    # set of parents for corners of the maintab (added on demand when GUI is built)
    self._tb_corners = {};
    #------ add page
    newpage = self.add_tool_button(Qt.TopLeftCorner,pixmaps.tab_new_raised.icon(),
        tooltip="open new page. You can also drop data items here.",
        class_=self.DataDropButton,
        click=self.add_page);
    newpage._dropitem = xcurry(MeqGUI.Grid.Services.addDataItem,_argslice=slice(0,1),newpage=True);
    QWidget.connect(newpage,PYSIGNAL("itemDropped()"),
        newpage._dropitem);
    #------ new panels button
    self._new_panel = self.add_tool_button(Qt.TopLeftCorner,pixmaps.view_right.icon(),
        tooltip="add more panels to this page. You can also drop data items here.",
        class_=self.DataDropButton,
        click=self._add_more_panels);
    self._new_panel._dropitem = xcurry(MeqGUI.Grid.Services.addDataItem,_argslice=slice(0,1),newcell=True);
    QWidget.connect(self._new_panel,PYSIGNAL("itemDropped()"),
        self._new_panel._dropitem);
    #------ align button
    self.add_tool_button(Qt.TopLeftCorner,pixmaps.view_split.icon(),
        tooltip="align panels on this page",
        click=self._align_grid);
    #------ remove page
    self.add_tool_button(Qt.TopRightCorner,pixmaps.tab_remove.icon(),
        tooltip="remove this page",
        click=self.remove_current_page);
    # init first page
    self.add_page();

  def show_message (self,message,error=False,timeout=2000):
    from MeqGUI.GUI import app_proxy_gui
    category = app_proxy_gui.Logger.Error if error else app_proxy_gui.Logger.Normal;
    self.wtop().emit(PYSIGNAL("showMessage"),message,None,category,timeout);

  # adds a tool button to one of the corners of the workspace viewer
  def add_tool_button (self,corner,icon,tooltip=None,click=None,
                        leftside=False,class_=QToolButton):
    # create corner box on demand
    layout = self._tb_corners.get(corner,None);
    if not layout:
      parent = QWidget(self._maintab);
      self._tb_corners[corner] = layout = QHBoxLayout(parent);
      layout.setMargin(2);
      self._maintab.setCornerWidget(parent,corner);
    # add button
    button = class_(layout.parentWidget());
    button._gw = weakref.proxy(self);
    if leftside:
      layout.insertWidget(0,button);
    else:
      layout.addWidget(button);
    button.setIcon(icon);
    button.setAutoRaise(True);
    if tooltip:
      button.setToolTip(tooltip);
    if callable(click):
      QWidget.connect(button,SIGNAL("clicked()"),click);
    return button;

  def wtop (self):
    return self._maintab;

  def show (self,shown=True):
    self.wtop().setShown(shown);
    self.wtop().emit(SIGNAL("shown"),shown,);

  def hide (self):
    self.wtop().hide();
    self.wtop().emit(SIGNAL("shown"),False,);

  def isVisible (self):
    return self.wtop().isVisible();

  def add_page (self,name=None,icon=None):
    page = MeqGUI.Grid.Page(self,self._maintab,max_nx=self.max_nx,max_ny=self.max_ny);
    wpage = page.wtop();
    wpage._page = page;
    _dprint(2,'name',name,'icon',icon);
    index = self._maintab.count();
    # generate page name, if none is supplied
    if name is None:
      name = 'Page %d'%(index+1);
      page.set_name(name,True); # auto_name = True
    else:
      page.set_name(name);
    page.set_icon(icon);
    # add page to tab
    icon = icon or QIcon();
    _dprint(2,'addTab:',name,icon);
    self._maintab.addTab(wpage,icon,name);
    self._maintab.setCurrentIndex(index);
    QWidget.connect(wpage,PYSIGNAL("layoutChanged()"),self._set_layout_button);
    wpage._change_icon = curry(self._maintab.setTabIcon,index);
    QWidget.connect(wpage,PYSIGNAL("setIcon()"),wpage._change_icon);
    wpage._rename = curry(self._maintab.setTabText,index);
    QWidget.connect(wpage,PYSIGNAL("setName()"),wpage._rename);
    return page;

  def remove_current_page (self):
    ipage = self._maintab.currentIndex();
    page = self._maintab.currentWidget();
    page._page.clear();
    # if more than one page, then remove (else clear and rename only)
    if self._maintab.count()>1:
      self._maintab.removeTab(ipage);
    else:
      page._page.set_name("Page 1",True);
    # renumber remaining pages
    for i in range(ipage,self._maintab.count()):
      wpage = self._maintab.widget(i);
      page = wpage._page;
      if page.is_auto_name():
        page.set_name('Page '+str(i+1),True);

  def current_page (self):
    return self._maintab.currentWidget()._page;

  def _align_grid (self):
    self.current_page().rearrange_cells();
    self.current_page().align_layout();
  def _add_more_panels (self):
    _dprint(5,"adding more panels");
    self.current_page().next_layout();
  def _set_layout_button (self,*dum):
    page = self.current_page();
    (nlo,nx,ny) = page.current_layout();
    self._new_panel.setDisabled(nlo >= page.num_layouts());
  def clear (self):
    _dprint(5,'GriddedWorkspace: clearing');
    self._maintab.widget(0)._page.clear();
    while self._maintab.count() > 1:
      page = self._maintab.widget(self._maintab.count()-1);
      page._page.clear();
      self._maintab.removeTab(self._maintab.count()-1);

  def set_current_page (self,page):
    self._maintab.setCurrentWidget(page.wtop());

  def allocate_cells (self,nrow=1,ncol=1,position=None,avoid_pos=None,
                           newcell=False,newpage=False,udi=''):
    """allocates cell or block of cells""";
    # is an explicit cell specified?
    if position:
      (page,x,y) = position;
      if isinstance(page,MeqGUI.Grid.Page):
        self.set_current_page(page);
      else:  # if page specified, create new one, else use current
        if page:
          page = self.add_page();
        else:
          page = self.current_page();
      return page.alloc_cells((x,y),nrow=nrow,ncol=ncol);
    # no, do we need a new page?
    elif newpage:
      self.add_page();
    # find suitable cell(s) and return
    if avoid_pos is not None:
      if avoid_pos[0] is self.current_page():
        avoid_pos = avoid_pos[1:];
      else:
        avoid_pos = None;
    return self.current_page().find_cells(udi,avoid_pos=avoid_pos,new=newcell,nrow=nrow,ncol=ncol) \
            or self.add_page().find_cells(udi,nrow=nrow,ncol=ncol);



#/usr/bin/python
#
#% $Id$ 
#
#
# Copyright (C) 2002-2007
# The MeqTree Foundation & 
# ASTRON (Netherlands Foundation for Research in Astronomy)
# P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>,
# or write to the Free Software Foundation, Inc., 
# 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

from MeqGUI.GUI import meqgui
from MeqGUI.GUI.pixmaps import pixmaps

from PyQt4.Qt import *
from Kittens.widgets import PYSIGNAL


class ConnectMeqKernel(QDialog):
    def __init__(self,parent=None,name=None,modal=0,fl=None):
        if fl is None:
          fl = Qt.WType_TopLevel|Qt.WStyle_Customize;
          fl |= Qt.WStyle_DialogBorder|Qt.WStyle_Title;
        
        QDialog.__init__(self,parent,name,modal,fl)
        
        self._default_path = "meqserver";
        
        self.image0 = pixmaps.trees48x48.pm();
        if not name:
            self.setName("ConnectDialog")

        # self.setSizeGripEnabled(0)
        lo_dialog = QVBoxLayout(self);

        LayoutWidget = QWidget(self,"lo_top")
        lo_dialog.addWidget(LayoutWidget);
        LayoutWidget.setSizePolicy(QSizePolicy.MinimumExpanding,QSizePolicy.MinimumExpanding);
        # LayoutWidget.setGeometry(QRect(10,10,472,400))
        lo_top = QVBoxLayout(LayoutWidget,11,6,"lo_top")

        lo_title = QHBoxLayout(None,0,6,"lo_title")

        self.title_icon = QLabel(LayoutWidget,"title_icon")
        self.title_icon.setSizePolicy(QSizePolicy(QSizePolicy.Fixed,QSizePolicy.Fixed,self.title_icon.sizePolicy().hasHeightForWidth()))
        self.title_icon.setPixmap(self.image0)
        self.title_icon.setAlignment(QLabel.AlignCenter)
        lo_title.addWidget(self.title_icon)

        self.title_label = QLabel(LayoutWidget,"title_label")
        lo_title.addWidget(self.title_label)
        lo_top.addLayout(lo_title)

        self.bg_connect = QButtonGroup(LayoutWidget,"bg_connect")
#        self.bg_connect.setInsideMargin(10);

        lo_connect = QVBoxLayout(self.bg_connect,11,6,"lo_connect")
        lo_connect_space = QSpacerItem(20,20,QSizePolicy.Minimum,QSizePolicy.Fixed)
        lo_connect.addItem(lo_connect_space)

        # self.btn_wait = QRadioButton(self.bg_connect,"btn_wait")
        # self.btn_wait.setChecked(1)
        # lo_connect.addWidget(self.btn_wait)

        self.btn_start = QRadioButton(self.bg_connect,"btn_start")
        self.btn_start.setChecked(1)
        lo_connect.addWidget(self.btn_start)

        lo_start_grp = QWidget(self.bg_connect)
        lo_start = QHBoxLayout(lo_start_grp,0,6,"lo_start")
        lo_start_space = QSpacerItem(20,20,QSizePolicy.Fixed,QSizePolicy.Minimum)
        lo_start.addItem(lo_start_space)
        
        lo_start_lbl = QLabel("program:",lo_start_grp);
        lo_start.addWidget(lo_start_lbl)
        self.start_pathname = QLineEdit(lo_start_grp,"start_pathname")
        self.start_pathname.setText(self._default_path);
        self.start_pathname.setSizePolicy(QSizePolicy.MinimumExpanding,QSizePolicy.Fixed);
        self.start_pathname.setMinimumSize(QSize(400,0))
        lo_start.addWidget(self.start_pathname)

        lo_start_grp2 = QWidget(self.bg_connect)
        lo_start2 = QHBoxLayout(lo_start_grp2,0,6,"lo_start2")
        lo_start_space2 = QSpacerItem(20,20,QSizePolicy.Fixed,QSizePolicy.Minimum)
        lo_start2.addItem(lo_start_space2)
        lo_start_lbl2 = QLabel("args:",lo_start_grp2);
        lo_start2.addWidget(lo_start_lbl2)
        self.start_args = QLineEdit(lo_start_grp2,"start_args")
        lo_start2.addWidget(self.start_args)
        self.start_browse = QPushButton(lo_start_grp2,"start_browse")
        lo_start2.addWidget(self.start_browse)
        self.start_default = QPushButton(lo_start_grp2,"start_default")
        self.start_default.setEnabled(False);
        lo_start2.addWidget(self.start_default)

        lo_connect.addWidget(lo_start_grp)
        lo_connect.addWidget(lo_start_grp2)
        
        # lo_start_grp.setEnabled(False);
        # lo_start_grp2.setEnabled(False);

        self.btn_remote = QRadioButton(self.bg_connect,"btn_remote")
        self.btn_remote.setEnabled(0)
        lo_connect.addWidget(self.btn_remote)
        
        lo_remote_grp = QWidget(self.bg_connect)
        lo_remote = QHBoxLayout(lo_remote_grp,0,6,"lo_remote")
        lo_remote_space = QSpacerItem(20,20,QSizePolicy.Fixed,QSizePolicy.Minimum)
        lo_remote.addItem(lo_remote_space)

        self.remote_host_lbl = QLabel(lo_remote_grp,"remote_host_lbl")
        self.remote_host_lbl.setAlignment(QLabel.AlignVCenter | QLabel.AlignRight)
        lo_remote.addWidget(self.remote_host_lbl)

        self.remote_host = QLineEdit(lo_remote_grp,"remote_host")
        lo_remote.addWidget(self.remote_host)

        self.remote_port_lbl = QLabel(lo_remote_grp,"remote_port_lbl")
        self.remote_port_lbl.setAlignment(QLabel.AlignVCenter | QLabel.AlignRight)
        lo_remote.addWidget(self.remote_port_lbl)

        self.remote_port = QLineEdit(lo_remote_grp,"remote_port")
        self.remote_port.setSizePolicy(QSizePolicy.Minimum,QSizePolicy.Fixed);
        self.remote_port.setMinimumSize(QSize(60,0))
        lo_remote.addWidget(self.remote_port)
        lo_connect.addWidget(lo_remote_grp)
        lo_remote_grp.setEnabled(False);
        
        lo_top.addWidget(self.bg_connect)

        lo_mainbtn = QHBoxLayout(None,0,6,"lo_mainbtn")
        lo_mainbtn_space = QSpacerItem(20,20,QSizePolicy.Expanding,QSizePolicy.Minimum)
        lo_mainbtn.addItem(lo_mainbtn_space)

        self.btn_ok = QPushButton(LayoutWidget,"btn_ok")
        self.btn_ok.setSizePolicy(QSizePolicy(QSizePolicy.Fixed,QSizePolicy.Fixed,1,0,self.btn_ok.sizePolicy().hasHeightForWidth()))
        self.btn_ok.setMinimumSize(QSize(60,0))
        self.btn_ok.setAutoDefault(1)
        self.btn_ok.setDefault(1)
        lo_mainbtn.addWidget(self.btn_ok)

        self.btn_cancel = QPushButton(LayoutWidget,"btn_cancel")
        self.btn_cancel.setSizePolicy(QSizePolicy(QSizePolicy.Fixed,QSizePolicy.Fixed,1,0,self.btn_cancel.sizePolicy().hasHeightForWidth()))
        self.btn_cancel.setMinimumSize(QSize(60,0))
        self.btn_cancel.setAutoDefault(1)
        lo_mainbtn.addWidget(self.btn_cancel)
        lo_space = QSpacerItem(20,20,QSizePolicy.Fixed,QSizePolicy.Expanding);
        lo_top.addItem(lo_space)
        lo_top.addLayout(lo_mainbtn)

        self.languageChange()
        
        LayoutWidget.adjustSize();

        #LayoutWidget.resize(QSize(489,330).expandedTo(LayoutWidget.minimumSizeHint()))
        #self.resize(QSize(489,330).expandedTo(self.minimumSizeHint()))
        self.clearWState(Qt.WState_Polished)
        
        self.connect(self.btn_ok,SIGNAL("clicked()"),self.accept)
        self.connect(self.btn_cancel,SIGNAL("clicked()"),self.reject)
        
        ### my additions
        self.connect(self.btn_start,SIGNAL("toggled(bool)"),lo_start_grp,SLOT("setEnabled(bool)"));
        self.connect(self.btn_start,SIGNAL("toggled(bool)"),lo_start_grp2,SLOT("setEnabled(bool)"));
        self.connect(self.btn_remote,SIGNAL("toggled(bool)"),lo_remote_grp,SLOT("setEnabled(bool)"));
        self.connect(self.start_browse,SIGNAL("clicked()"),self.browse_kernel_dialog);
        self.connect(self.start_default,SIGNAL("clicked()"),self.reset_default_path);
        self.connect(self.start_pathname,SIGNAL("textChanged(const QString &)"),self.changed_path);


    def languageChange(self):
        self.setWindowTitle(self.__tr("Connect to meqserver"))
        self.title_icon.setText(QString.null)
        self.title_label.setText(self.__tr( \
          """<p>Not connected to a MeqTrees kernel.</p>
          <p><i>If you will be starting a kernel locally using external tools,
          a connection should be established automatically.</i></p>
          <p><i>Otherwise, select one of the connection methods below.</i></p>""" \
          ));

        self.bg_connect.setTitle(self.__tr("Pick a connection method"))
        # self.btn_wait.setText(self.__tr("wait for local connection"))
        self.btn_start.setText(self.__tr("start a local meqserver:"))
        self.start_browse.setText(self.__tr("Browse..."))
        self.start_default.setText(self.__tr("Reset"))
        self.btn_remote.setText(self.__tr("connect to remote meqserver:"))
        self.remote_host_lbl.setText(self.__tr("Host"))
        self.remote_port_lbl.setText(self.__tr("Port"))
        self.remote_port.setInputMask(self.__tr("#####; "))
        self.btn_ok.setText(self.__tr("&OK"))
        self.btn_ok.setAccel(QString.null)
        self.btn_cancel.setText(self.__tr("&Cancel"))
        self.btn_cancel.setAccel(QString.null)


    def __tr(self,s,c = None):
        return qApp.translate("ConnectMeqKernel",s,c)

    ### override the accept method to start kernel as appropriate
    def accept (self):
      selected = self.bg_connect.selected();
      if selected is self.btn_start:
        # start kernel
        pathname = str(self.start_pathname.text());
        args = str(self.start_args.text());
        self.emit(SIGNAL("startKernel"),pathname,args);
      elif selected is self.btn_remote:
        # not implemented yet
        pass;
      QDialog.accept(self);
      
    def set_default_args (self,args):
      self.start_args.setText(args);
    
    def set_default_path (self,path):
      self._default_path = path;
      self.start_pathname.setText(path);
      self.start_default.setEnabled(False);

    def reset_default_path (self):
      self.start_pathname.setText(self._default_path);
      self.start_default.setEnabled(False);
      
    def changed_path (self,path):
      self.start_default.setEnabled(bool(str(path) != self._default_path));

    def browse_kernel_dialog (self):
      try: dialog = self._browse_dialog;
      except AttributeError:
        self._browse_dialog = dialog = QFileDialog(self,"Select meqserver binary");
        dialog.resize(500,dialog.height());
        dialog.setMode(QFileDialog.ExistingFile);
        dialog.setViewMode(QFileDialog.Detail);
      else:
        # trying to have the same effect as rereadDir() in the Qt3 version...
        dialog.setDirectory(dialog.directory());
      if dialog.exec_() == QDialog.Accepted:
        self.start_pathname.setText(str(dialog.selectedFiles()[0]));
        



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#
#% $Id$
#
#
# Copyright (C) 2002-2007
# The MeqTree Foundation &
# ASTRON (Netherlands Foundation for Research in Astronomy)
# P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>,
# or write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

MainApp = None;
MainAppThread = None;

from Timba.dmi import *
from MeqGUI import qt_threading
from MeqGUI.GUI.pixmaps import pixmaps
from Timba import dmi_repr
from MeqGUI import Grid
from MeqGUI.GUI.browsers import *
from MeqGUI.GUI.widgets import *

import sys
import time

from PyQt4.Qt import *
from Kittens.widgets import PYSIGNAL

import traceback
import weakref
import re
import imp
import signal

dmirepr = dmi_repr.dmi_repr();

_dbg = verbosity(0,name='appgui');
_dprint = _dbg.dprint;
_dprintf = _dbg.dprintf;

_MessageCategories = {};

# Reloads GUI modules on the fly
# Does not work reliably, so:
_reloading_enabled = False;

_reloadables = set([ __name__,'app_pixmaps','dmi_repr','app_browsers','gridded_workspace','array_plotter']);
def reloadableModule (name):
  _reloadables.add(name);

def reloadAllModules ():
  imp.acquire_lock();
  try:
    for name in _reloadables:
      try:
        info = imp.find_module(name);
      except ImportError as what:
        print('skipping module',name,':',what);
      else:
        print('reloading module:',info[1]);
        imp.load_module(*((name,)+info));
  finally:
    imp.release_lock();

def defaultFont ():
  global _def_font;
  try: return _def_font;
  except NameError:
    _def_font = QApplication.font();
  return _def_font;

def defaultBoldFont ():
  global _def_bold_font;
  try: return _def_bold_font;
  except NameError:
    _def_bold_font = QFont(defaultFont());
    _def_bold_font.setBold(True);
  return _def_bold_font;

def defaultCursor ():
  global _def_default_cursor;
  try: return _def_default_cursor;
  except NameError:
    _def_default_cursor = QCursor(Qt.ArrowCursor);
  return _def_default_cursor;

def hourglassCursor ():
  global _def_hourglass_cursor;
  try: return _def_hourglass_cursor;
  except NameError:
    _def_hourglass_cursor = QCursor(Qt.WaitCursor);
  return _def_hourglass_cursor;

def setBusyCursor ():
  QApplication.setOverrideCursor(hourglassCursor());

def restoreCursor ():
  QApplication.restoreOverrideCursor();

def busyCursorMethod (func):
  def callit (*args,**kw):
    try:
      setBusyCursor();
      func(*args,**kw);
    finally: restoreCursor();
  return callit;

class Logger(HierBrowser):
  Normal = 0;
  Event  = 1;
  Error  = 2;
#  _LogPixmaps =  { Normal:pixmaps.gear, Error:pixmaps.round_red_cross };
  _LogPixmaps =  { Error:pixmaps.red_round_cross };
  _LogCatNames = { Normal:"message",Event:"event",Error:"error" };
  def __init__(self,parent,name,
               click=None,udi_root=None,
               enable=True,scroll=False,use_clear=True,auto_clear=True,
               limit=-100):
    """Initializes a Logger panel. Arguments are:
          parent:     parent widget
          name:       name of panel (appears at top)
          limit:      initial log size. If <0, then the log size control starts
                      out disabled. If None, size is always unlimited.
          enable:     initial state of enable control (None for no control)
          scroll:     initial state of scroll control (None for no control)
          use_clear:  if True, logger has a clear button
          auto_clear: initial state of auto-clear toggle (None for no toggle)
          click:      callback, called when a log item is clicked
                      (QListView::mouseButtonClicked() signal is connected to this slot)
          udi_root:   the UDI root name corresponding to this logger.
                      if None, then panel name is used instead.
    """;
    self._vbox = QWidget(parent);
    self._vbox_lo = QVBoxLayout(self._vbox);
    self._vbox_lo.setContentsMargins(0,0,0,0);
    self._vbox_lo.setSpacing(0);
    # init the browser base class
    HierBrowser.__init__(self,self._vbox,name,udi_root=udi_root);
    self._vbox_lo.addWidget(self.treeWidget());
    # add controls
    self._controlgrid = QWidget(self._vbox);
    self._vbox_lo.addWidget(self._controlgrid);
    self._controlgrid_lo = QGridLayout(self._controlgrid); # ,3,6);
    self._controlgrid_lo.setColumnStretch(1,1000);
    for icol in 3,5:
      self._controlgrid_lo.setColumnMinimumWidth(icol,8);
    self._controlgrid_lo.setContentsMargins(0,0,0,0);
    self._controlgrid_lo.setSpacing(0);
    self.enabled = enable != False;
    # enable control
    if enable is not None:
      self._enable = QCheckBox("log",self._controlgrid);
      self._enable.setChecked(enable);
      QObject.connect(self._enable,SIGNAL('toggled(bool)'),self._toggle_enable);
      self._controlgrid_lo.addWidget(self._enable,1,2);
    else:
      self._enable = None;
    # scroll control
    if scroll is not None:
      self._scroll = QCheckBox("scroll",self._controlgrid);
      self._scroll.setChecked(scroll);
      self._controlgrid_lo.addWidget(self._scroll,2,2);
    else:
      self._scroll = None;
    # limit control
    if limit is not None:
      self._limit_enable = QCheckBox("limit",self._controlgrid);
      self._limit_field  = QLineEdit("",self._controlgrid);
      self._limit_field.setFixedWidth(60);
      try: self._limit_field.setInputMask('00000');
      except: pass; # older Qt versions do not support this
      self.wtop().connect(self._limit_enable,SIGNAL('toggled(bool)'),
                      self._limit_field,SLOT('setEnabled(bool)'));
      self.wtop().connect(self._limit_field,SIGNAL('returnPressed()'),
                      self._enter_log_limit);
      self._controlgrid_lo.addWidget(self._limit_enable,1,4);
      self._controlgrid_lo.addWidget(self._limit_field,2,4);
    else:
      self._limit_enable = None;
    # auto-clear control
    if auto_clear is not None:
      self._auto_clear = QCheckBox("autoclear",self._controlgrid);
      self._auto_clear.setChecked(auto_clear);
      self._controlgrid_lo.addWidget(self._auto_clear,1,6);
    else:
      self._auto_clear = None;
    # clear button
    if use_clear:
      clear = QPushButton("Clear",self._controlgrid);
      QObject.connect(clear,SIGNAL("clicked()"),self.clear);
      self._controlgrid_lo.addWidget(clear,2,6);
    # connect click callback
    if callable(click):
      QObject.connect(self.treeWidget(),SIGNAL('mouseButtonClicked()'),click);
    # event counter
    self._event_count = 0;
    # set log limit
    self.set_log_limit(limit);
    self._disable_until = 0;
    self._prev_event_clock = 0;
    self._flood_clock = 0;
    self._flood_count = 0;
    # compile regex to match our udi pattern
    self._patt_udi = re.compile("/"+self._udi_root+"/(.*)$");
    # define get_drag_item methods for the listview
    self.treeWidget().get_drag_item = self.get_drag_item;
    self.treeWidget().get_drag_item = self.get_drag_item;
    self.treeWidget().header().hide();

  def event_count (self):
    return self._event_count;

  def wtop (self):
    return self._vbox;

  def enable (self,val=True):
    self._enable.setChecked(val);

  def disable (self,val=True):
    self._enable.setChecked(not val);

  def connected (self,conn=True):
    if conn and self._auto_clear is not None and self._auto_clear.isChecked():
      self.clear();

  def _enter_log_limit (self):
    try: self._limit = int(str(self._limit_field.text()));
    except: pass; # catch conversion errors
    self._limit = max(10,self._limit);
    self._limit_field.setText(str(self._limit));
    self.apply_limit(self._limit);

  def set_log_limit (self,limit):
    if limit is None:
      self._limit = None;
    else:
      self._limit = abs(limit);
      if self._limit_enable is not None:
        self._limit_field.setText(str(self._limit));
        self._limit_enable.setChecked(limit>0);
        self._limit_field.setEnabled(limit>0);
      self.apply_limit(self._limit);

  def add (self,msg,label=None,content=None,
           category=Normal,force=False,
           flood_protection=50,
           udi=None,udi_key=None,name=None,caption=None,desc=None,viewopts={}):
    """Adds item to logger. Arguments are:
      msg:     item message (for message column)
      label:   item label (for label column -- timestamp is used if this is None)
      content: item data content (item will be expandable if this is not None)
      category: item category: Normal, Event, Error.
      force:   if False and logging is disabled, add() call does nothing.
               if True, item is always added.
      udi_key: item UDI key, auto-generated if None
      flood_protection: if !=0, logger will check the clock, and if more than 'flood_protection'
          events have been received in the past second, will suspend logging for 10 seconds, or until
          one eventless second has elapsed.
    If content is not None, then content will be available to viewers. In
    this case, the following parameters may define its properties:
      name:    item name for viewers; if None, then generated from udi
      caption: item caption for viewers; if None, then generated from udi
      desc:    item description; if None, then label is used
      viewopts: dict of optional viewer settings for this item
    Return value: a QListViewItem
    """
    # disabled? return immediately
    if not force and not self.enabled:
      return None;
    # check for event flood
    if flood_protection:
      clock = time.time();
      # if >3 sec since last event, always reenable the logger and restart the flood counter
      if clock - self._prev_event_clock >= 3:
        self._disable_until = 0;
        self._flood_clock = clock;
        self._flood_count = 0;
      self._prev_event_clock = clock;
      # if logging is temporarily disabled, return
      if clock < self._disable_until:
        return None;
      # count events for a one-second interval
      flood = self._flood_count > flood_protection;
      if clock - self._flood_clock < 1:
        self._flood_count += 1;
      else:
        self._flood_clock = clock;
        self._flood_count = 0;
      if flood:
        msg = "Event flood detected, disabling logging for 10 seconds";
        content = udi = udi_key = None;
        category = Logger.Error;
        self._disable_until = clock+10;
        self._flood_count = 0;
    # # is scrolling enabled?
    # preserve_top_item = self._scroll is not None and not self._scroll.isOn() and \
    #                    self.treeWidget().itemAt(QPoint(0,0));
    # if label not specified, use a timestamp
    self._event_count += 1;
    if label is None:
      label = time.strftime("%H:%M:%S");
    # if udi_key is None, set to the id type object. This will tell the
    # HierBrowser.Item constructor to use the item id for key, rather than
    # the text in column 0
    if udi is None and udi_key is None:
      udi_key = str(self._event_count);
    # create listview item
    item = self.Item(None,label,msg,udi=udi,udi_key=udi_key, \
      name=name,caption=caption,desc=desc or label);
    item._category = category;
    # if content is specified, cache it inside the item
    if content is not None:
      # if content is just a single message, override viewable property to False,
      # else let browser decide (viewable=None)
      viewable = None;
      if isinstance(content,dict) and \
         (len(content)==1 and list(content.keys())[0] in MessageCategories):
        viewable = False;
      item.cache_content(content,viewable=viewable,viewopts=viewopts);
    # add pixmap according to category
    pm = self._LogPixmaps.get(category,None);
    if pm is not None:
      item.setIcon(2,pm.icon());
    # apply a log limit
    if self._limit is not None:
      self.apply_limit(self._limit);
    # if scroll is enabled, ensure item is visible
    self.treeWidget().insertTopLevelItem(self.treeWidget().topLevelItemCount(),item)
    if item._udi:
      self.treeWidget()._content_map[item._udi] = item;
    if self._scroll is None or self._scroll.isChecked():
      self.treeWidget().scrollToItem(item);
    return item;

  def _toggle_enable (self,en):
    self.enabled = en;
    if en: self.add("logging enabled",category=self.Normal);
    else:  self.add("logging disabled",category=self.Error,force=True);

class EventLogger (Logger):
  def __init__(self,parent,name,evmask="*",*args,**kwargs):
    Logger.__init__(self,parent,name,scroll=True,*args,**kwargs);
    # label = QLabel('Event mask:',self._controlgrid);
    # self._controlgrid_lo.addWidget(label,0,0);
    self._evmask_field  = QLineEdit(str(evmask),self._controlgrid);
    self._controlgrid_lo.addWidget(self._evmask_field,0,0,2,6);
    self.wtop().connect(self._evmask_field,SIGNAL('returnPressed()'),
                        self._enter_mask);
    self.set_mask('*');

  def _enter_mask(self):
    self.set_mask(str(self._evmask_field.text()));

  def set_mask (self,mask):
    self._filters = [];
    maskstrings = [];
    for m in mask.split(';'):
      m.strip(); # strips whitespace
      invert = ( m[0] == '!' );
      if invert:
        m = m[1:];
      # try to convert mask to hiid, skip if fails
      try: hm = make_hiid(m);
      except: continue;
      # add a filter
      self._filters.append((invert,hm));
      if invert:
        maskstrings.append('!'+str(hm));
      else:
        maskstrings.append(str(hm));
    self._mask = ';'.join(maskstrings);
    self._evmask_field.setText(self._mask);
    self.wtop().emit(SIGNAL("maskChanged"),self.wtop(),self._mask);

  def get_mask (self):
    return self._mask;

  # for event viewers, use the event name as name, and 'event' as description
  def add (self,msg,*args,**kwargs):
    # apply filters
    for (invert,mask) in self._filters:
      if mask.matches(msg):
        if invert:
          return None;
        else:
          break;
    else: # matched nothing
      return None;
    # add entry
    msg = msg.lower();
    label = time.strftime("%H:%M:%S");
    kw = kwargs.copy();
    kw['udi_key'] = "%s:%d:%s" % (msg,self._event_count,time.strftime("%H%M%S"));
    kw['label'] = label;
    kw['name'] = "%s @%s" % (msg,label);
    kw['caption'] = "%s <small>%s</small>" % (msg,label);
    kw['desc'] = "event %s at %s" % (msg,label);
    Logger.add(self,msg,*args,**kw);

class MessageLogger (Logger):
  def __init__(self,*args,**kwargs):
    Logger.__init__(self,scroll=True,*args,**kwargs);
    self._num_err = 0;
    # self.wtop().connect(self._tw,SIGNAL('clicked(QTreeWidgetItem*)'),
    #                    self._clear_error_count);
    self._wclearerr = QPushButton(pixmaps.exclaim.icon(),"Clear errors",self._controlgrid);
    self._controlgrid_lo.addWidget(self._wclearerr,1,0);
    self._wclearerr.setVisible(False);
    QObject.connect(self._wclearerr,SIGNAL("clicked()"),self._clear_error_count);

  def add (self,msg,category=Logger.Normal,*args,**kwargs):
    label = time.strftime("%H:%M:%S");
    kw = kwargs.copy();
    kw['udi_key'] = "%d:%s" % (self._event_count,label);
    kw['name'] = "message @%s" % (label,);
    kw['caption'] = "message <small>%s</small>" % (label,);
    kw['desc'] = "message at %s" % (label,);
    item = Logger.add(self,msg,category=category,*args,**kw);
    # keep track of new errors
    if item and category is Logger.Error:
      if self._num_err == 0:
        self._first_err = item;
      self._num_err += 1;
      self._wclearerr.setVisible(True);
      self.wtop().emit(SIGNAL("hasErrors"),self.wtop(),self._num_err);
      self._last_err = item;
    return item;
  def _clear_error_count (self):
    self._num_err = 0;
    self._first_err = self._last_err = None;
    self._wclearerr.setVisible(False);
    self.wtop().emit(SIGNAL("hasErrors"),self.wtop(),0);
  def clear (self):
    Logger.clear(self);
    self._clear_error_count();


#--------------------------------------------------------------
#--- some custom event types
#--------------------------------------------------------------
EvType_AppMessage = QEvent.Type(QEvent.User + 1);
EvType_Callable   = QEvent.Type(QEvent.User + 2);

class QAppMessageEvent (QEvent):
  """This custom event class is used to pass a callable object into the main app thread."""
  def __init__ (self,msg,value,server):
    QEvent.__init__(self,EvType_AppMessage);
    self.msg = msg;
    self.value = value;
    self.server = server;

class QCallableEvent (QEvent):
  """This custom event class is used to pass a callable object into the main app thread."""
  def __init__ (self,func,*args,**kwargs):
    QEvent.__init__(self,EvType_Callable);
    self.func = func;
    self.args = args;
    self.kwargs = kwargs;
  def call (self):
    return self.func(*self.args,**self.kwargs);

#--------------------------------------------------------------
#--- app_proxy_gui() class
#--------------------------------------------------------------
class app_proxy_gui(verbosity,QMainWindow,utils.PersistentCurrier):
  def __init__(self,app,verbose=0,size=(500,500),poll_app=None,*args,**kwargs):
    """create and populate the main application window""";
    global gui;
    gui = self;
    #------ starts the main app object and event thread, if not already started
    self._qapp = mainapp();
    #------ init base classes
    verbosity.__init__(self,verbose,name=app.name()+"/gui");
    _dprint(1,"initializing");
    QMainWindow.__init__(self,*args);
    self.app = app;
    self._connected = False;

    self.populate(size=size,*args,**kwargs);

    #------ set size
    self.setCentralWidget(self.splitter);
    sz = self.size().expandedTo(QSize(size[0],size[1]));
    self.resize(sz);

    # events from remote application are emitted as PYSIGNALS taking
    # two arguments. Connect some standard handlers here
    #QObject.connect(self,PYSIGNAL("hello"),self._attached_server_event);
    #QObject.connect(self,PYSIGNAL("hello"),self.xcurry(self._update_app_state));
    #QObject.connect(self,PYSIGNAL("bye"),self._detached_server_event);
    #QObject.connect(self,PYSIGNAL("bye"),self.xcurry(self._update_app_state));
    QObject.connect(self,PYSIGNAL("app.notify.state"),self.xcurry(self._update_app_state));

    #------ start timer when in polling mode
    if poll_app:
      self.startTimer(poll_app);

    global splash_screen;
    if splash_screen is not None:
      splash_screen.finish(self);

    _dprint(2,"init complete");\

  class PanelizedWindow (QWidget):
    def __init__ (self,parent,name,shortname,icon,*args):
      QWidget.__init__(self,parent,*args);
      self._lo = QVBoxLayout(self);
      self._lo.setSpacing(0);
      self._lo.setContentsMargins(2,0,2,0);
      self.name = name;
      self.shortname = shortname;
      self.icon = icon;
      # build title "toolbar"
      titlebar = QFrame(self);
      self._lo.addWidget(titlebar);
      titlebar.setLineWidth(0);
      titlebar.setFrameStyle(QFrame.Panel|QFrame.Raised);
      tblo = QHBoxLayout(titlebar);
      tblo.setMargin(2);
      self.populate_titlebar(titlebar,tblo);
      # separator
      sep = QFrame(self);
      sep.setFrameStyle(QFrame.HLine+QFrame.Sunken);
      self._lo.addWidget(sep);
    def populate_titlebar (self,titlebar,layout):
      icon = QLabel(titlebar);
      pm = self.icon.pixmap(128,128);
      height = pm.height() + 4;
      icon.setPixmap(pm);
      icon.setMargin(0);
      icon.setSizePolicy(QSizePolicy.Minimum,QSizePolicy.Minimum);
      icon.setFixedSize(height,height);
      label = QLabel("<b>"+self.name+"</b>",titlebar);
      label.setMargin(0);
      label.setSizePolicy(QSizePolicy.Expanding,QSizePolicy.Minimum);
      label.setFixedHeight(height);
      minbtn = QToolButton(titlebar);
      minbtn.setIcon(pixmaps.minimize_line.icon());
      minbtn.setAutoRaise(True);
      minbtn.setFixedSize(height,height);
      minbtn.setToolTip("Minimize this panel");
      QObject.connect(minbtn,SIGNAL("clicked()"),self.hide);
      layout.addSpacing(2);
      layout.addWidget(icon);
      layout.addSpacing(2);
      layout.addWidget(label);
      layout.addWidget(minbtn);
    def resizeEvent (self,ev):
      self.emit(SIGNAL("resized"));
      return QWidget.resizeEvent(self,ev);
    def setVisible (self,visible):
      _dprint(1,self.name,"visible",visible);
      self.emit(SIGNAL("visible"),visible);
      if visible:
        _dprint(1,'showing',self,self.parent());
        self.emit(SIGNAL("shown"));
      else:
        _dprint(1,'hiding',self,self.parent());
        self.emit(SIGNAL("hidden"));
      QWidget.setVisible(self,visible);
    def visQAction (self,parent=None):
      try: return self._qa_vis;
      except AttributeError: pass;
      qa = self._qa_vis = QAction(self.icon,"Show/hide "+self.name,parent);
      qa.setCheckable(True);
      qa.setChecked(self.isVisible());
      QObject.connect(qa,SIGNAL("triggered(bool)"),self.setVisible);
      QObject.connect(self,PYSIGNAL("visible()"),qa.setChecked);
      return qa;
    def makeMinButton (self,parent):
      return self.MinimizedPanelButton(self,parent);
    def addWidget (self,widget):
      self._lo.addWidget(widget);

    class MinimizedPanelButton (QToolButton):
      def __init__ (self,panel,parent):
        QToolButton.__init__(self,parent);
        self._panel = panel;
        self._label = [ "("+panel.shortname+")",panel.shortname+" " ];
        self.setIcon(panel.icon);
        self.setText(self._label[int(panel.isVisible())]);
        self.setToolButtonStyle(Qt.ToolButtonTextBesideIcon);
        self.setToolTip("Show "+panel.name);
        QObject.connect(self,SIGNAL("clicked()"),self._toggle);
        QObject.connect(panel,PYSIGNAL("visible()"),self._show_panel);
        self._default_palette = self.palette();
        self._flash_palette   = QPalette(self.palette());
        self._flash_palette.setBrush(QPalette.Background,QBrush(QColor("yellow")));
      def _toggle (self):
        self._panel.setVisible(not self._panel.isVisible());
      def _show_panel (self,show):
        if show:
          self.setText(self._label[1]);
          self.setFixedWidth(self.width());
        else:
          self.setText(self._label[0]);
          self.setPalette(self._flash_palette);
          QTimer.singleShot(300,self._unsetPalette);
      def _unsetPalette (self):
        self.setPalette(self._default_palette);

  def populate (self,main_parent=None,*args,**kwargs):
    #------ main window contains a splitter
    splitter = self.splitter = QSplitter(Qt.Horizontal,main_parent or self);
    # splitter.setFrameStyle(QFrame.Box+QFrame.Plain);
    splitter.setChildrenCollapsible(True);

    #------ create top-level tab bar
    self.maintab_panel = self.PanelizedWindow(splitter,"Tabbed Tools","Tabs",pixmaps.tabs.icon());
    self.maintab = maintab = QTabWidget(self.maintab_panel);
    self.maintab_panel.addWidget(maintab);
    self.connect(self.maintab,SIGNAL("currentChanged(int)"),self._change_current_page);
    maintab.setTabPosition(QTabWidget.North);
    splitter.setStretchFactor(splitter.indexOf(self.maintab_panel),0);
    _dprint(1,"parent is",self.maintab_panel.parent());

    #------ create a message log
    self.msglog = MessageLogger(self,"message log",enable=None,limit=1000,
          udi_root='message');
    self.msglog.add('start of log',category=Logger.Normal);
    QObject.connect(self.msglog.wtop(),PYSIGNAL("cleared()"),
                    self.curry(self._reset_maintab_label,self.msglog.wtop()));
    QObject.connect(self.msglog.wtop(),PYSIGNAL("hasErrors()"),self._indicate_msglog_errors);
    QObject.connect(self.msglog.treeWidget(),PYSIGNAL("displayDataItem()"),self.display_data_item);
    QObject.connect(self,PYSIGNAL("isConnected()"),self.msglog.connected);
    # set current page to message log
    self._current_page = self.msglog.wtop();
    self.add_tab(self.msglog.wtop(),"Messages");
    self.msglog.wtop()._error_label = "%d errors";
    self.msglog.wtop()._error_icon = pixmaps.exclaim.icon();
    self._last_log_message = time.clock();
    
    #------ create an event log
    self.eventlog = EventLogger(self,"event log",limit=1000,evmask="*",
          udi_root='event');
    QObject.connect(self.eventlog.treeWidget(),PYSIGNAL("displayDataItem()"),self.display_data_item);
    QObject.connect(self,PYSIGNAL("isConnected()"),self.eventlog.connected);

    self.eventtab = QTabWidget(self.maintab);
    self.eventtab.setTabPosition(QTabWidget.South);
    self.add_tab(self.eventtab,"Events");
    QObject.connect(self.eventtab._show_qaction,SIGNAL("toggled(bool)"),self.eventlog.enable);

    #------ event window tab bar
    self.eventtab.setTabShape(QTabWidget.Triangular);
    self.eventtab.addTab(self.eventlog.wtop(),"*");
    self.connect(self.eventlog.wtop(),PYSIGNAL("maskChanged()"),self._change_eventlog_mask);

    #------ status bar, pause button
    self.statusbar = self.statusBar();
    ## self.pause_button = QToolButton(self.statusbar);
    ## self.pause_button.setAutoRaise(True);
    statholder = QWidget(self.statusbar);
    statholder_lo = QHBoxLayout(statholder);
    statholder_lo.setMargin(2);
    self.status_icon  = QLabel(statholder);
    self.status_label = QLabel(statholder);
    statholder_lo.addWidget(self.status_icon);
    statholder_lo.addWidget(self.status_label);
    # self.status_icon.setFrameStyle(QFrame.NoFrame);
    self.status_icon.setMinimumWidth(20);
    self.status_icon.setMaximumWidth(20);
    self.status_icon.setAlignment(Qt.AlignCenter);


    ##    self.pause_button.setIconSet(pixmaps.pause_normal.icon());
    ##    self.pause_button.setToolTip("pause the application");
    ##    self.pause_button.setDisabled(True);
    ##    self.connect(self.pause_button,SIGNAL("clicked()"),self._press_pause);
    ##    self.pause_requested = None;

    #------ reload button
    if _reloading_enabled:
      reloadbtn = QToolButton(self.statusbar);
      reloadbtn.setIcon(pixmaps.reload_slick.icon());
      reloadbtn.setToolTip("reload Python modules");
      self.connect(reloadbtn,SIGNAL("clicked()"),reloadAllModules);

    # self.status_icon.setFrameStyle(QFrame.NoFrame);
    ## self.statusbar.addWidget(self.pause_button,0,True);
    if _reloading_enabled:
      self.statusbar.addWidget(reloadbtn,0,True);
    self.statusbar.addWidget(statholder);

    #------ gridded workspace
    self.gw_panel = self.PanelizedWindow(splitter,"Gridded Viewers","Grid",pixmaps.view_split.icon());
    self.gw = gw = Grid.Workspace(self.gw_panel,max_nx=4,max_ny=4);
    self.gw_panel.addWidget(self.gw.wtop());
    splitter.setStretchFactor(splitter.indexOf(self.gw_panel),1);
    self.gw_panel.hide();
    QObject.connect(self.gw.wtop(),PYSIGNAL("shown()"),self.gw_panel.setShown);
    QObject.connect(self.gw.wtop(),PYSIGNAL("showMessage"),self.log_message);

    self.gw_visible = {};
#    gw.add_tool_button(Qt.TopRight,pixmaps.remove.pm(),
#      tooltip="hide the value browser panel",click=self.hide_gridded_workspace);
    Grid.Services.setDefaultWorkspace(self.gw);
    # QObject.connect(self.gw.wtop(),PYSIGNAL("shown()"),self._gridded_workspace_shown);

#     self.show_workspace_button = DataDroppableWidget(QToolButton)(maintab);
#     self.show_workspace_button.setPixmap(pixmaps.view_split.pm());
#     self.show_workspace_button.setAutoRaise(True);
#     maintab.setCornerWidget(self.show_workspace_button,Qt.BottomRight);
#     QObject.connect(self.show_workspace_button,SIGNAL("clicked()"),self.gw.show);
#     QObject.connect(self.show_workspace_button,PYSIGNAL("itemDropped()"),
#                     self.xcurry(self.display_data_item,_argslice=slice(0,1)));
#     self.show_workspace_button.setToolTip("show the viewer panel. You can also drop data items here.");

    splitter.setSizes([200,600]);
##    self.maintab.setCornerWidget(self.pause_button,Qt.TopRight);

  def show(self):
    #------ show the main window
    _dprint(2,"showing GUI");
    self._update_app_state();
    QMainWindow.show(self);

  def add_tab (self,widget,label,icon=None,index=-1):
    widget._default_label = label;
    widget._default_icon  = icon = icon or QIcon();
    widget._default_index = index;
    widget._show_qaction = QAction(icon,label+" tab",self);
    widget._show_qaction.setCheckable(True);
    widget._show_qaction.setChecked(True);
    self.maintab.insertTab(index,widget,icon,label);
    QObject.connect(widget._show_qaction,SIGNAL("toggled(bool)"),
      self.curry(self.show_tab,widget));

  def rename_tab (self,widget,label,icon=None):
    widget._default_label = label;
    index = self.maintab.indexOf(widget);
    self.maintab.setTabText(index,label);
    if icon:
      widget._default_icon = icon;
      self.maintab.setTabIcon(index,icon);
    widget._show_qaction.setText(label);

  def show_tab (self,widget,show=True,switch=True):
    curindex = self.maintab.indexOf(widget);
    if show:
      if curindex<0:
        widget.show();
        self.maintab.insertTab(widget._default_index,widget,widget._default_icon,widget._default_label);
        widget._show_qaction.setChecked(True);
      if switch:
        self.maintab.setCurrentWidget(widget);
    elif not show and curindex >= 0:
      widget._default_index = curindex;
      widget.hide();
      self.maintab.removeTab(curindex);
      widget._show_qaction.setChecked(False);

  def show_gridded_workspace (self,shown=True):
    _dprint(1,"showing",shown);
    self.gw_panel.setShown(shown);
    self.gw.show(shown);

  def hide_gridded_workspace (self):
    return self.show_gridded_workspace(False);

##### slot: called when change-of-page occurs
  def _change_current_page (self,index):
    page = self.maintab.widget(index);
    if page is not self._current_page:
      # clears message from status bar whenever a tab changes
      self.statusbar.clearMessage();
      # emit signals
      self._current_page.emit(SIGNAL("leaving"));
      page.emit(SIGNAL("entering"));
      self._current_page = page;
#       # show or hide the workspace
#       if self.gw_visible.get(page,False):
#         self.gw.wtop().show();
#         self.show_workspace_button.hide();
#       else:
#         self.gw.wtop().hide();
#         self.show_workspace_button.show();

##### displays data item in gridded workspace
  def display_data_item (self,item,kwargs={}):
    Grid.Services.addDataItem(item,gw=self.gw,**kwargs);

##### event relay: reposts message as a Qt custom event for ourselves
  MessageEventType = QEvent.User+1;
  def _relay_event (self,event,value,server):
    _dprint(5,'_relay_event:',event,value);
    ev = QAppMessageEvent(event,value,server);
    QApplication.postEvent(self,ev);
    _dprint(5,'_relay_event: event posted');

##### event handler for timer messages
  def timerEvent (self,event):
    # check WP for messages
    self.app.poll();

##### Qt customEvent handler maps to handleAppEvent(). This is used to relay events
  def customEvent (self,event):
    self.handleAppEvent(event.msg,event.value,event.server);

##### event handler for app events from octopussy
  def handleAppEvent (self,ev,value,server):
    _dprint(5,'appEvent:',ev,value);
    # process server attach/detach events
    if ev == self.app.server_detach_event:
      self._detached_server_event(ev,value,server);
      return;
    elif ev == self.app.server_attach_event:
      self._attached_server_event(ev,value,server);
      return;
    # ignore events not from current server
    if server is not self.app.current_server:
      return;
    # ignore server_state events: they are doubled up
    if ev is self.app.server_state_event:
      return;
    try:
      report = False;
      msgtext = None;
      # see if event contains state updates, or a message to be logged
      if isinstance(value,record):
        # update app state if appropriate fields are found
        if hasattr(value,'app_state') or hasattr(value,'app_state_str'):
          self._update_app_state();
        # the value of 'content' will later determine whether a data object
        # is logged along with the message. This will be the case if the
        # content is anything more complicated than a single string. In
        # all these cases we will reassign to content
        content = None;
        # if there's something else besides the message, add it as hierarchical
        # content to the entry
        if len(value) > 1:
          content = value;
        # look for fields of specific form ('message', 'error', etc.)
        for (field,cat) in list(MessageCategories.items()):
          msg = value.get(field,None);
          if msg is None:
            continue;
          # if msg is a sequence, reverse the order to form up message text
          # if not, make sure it is treated as one anyway
          if isinstance(msg,(list,tuple)):
            content = content or msg;
            msg = msg[-1::-1];
          else:
            msg = [msg];
          text = [];
          # now iterate to collect complete text
          for m in msg:
            # first form: message field is a string
            if isinstance(m,str):
              text.append(m);
            # second form: message field is a record containing 'message' field
            elif isinstance(m,record):
              content = content or m;
              try: text.append(m.message);
              except: pass;
          # have we collected any messages?
          if text:
            if len(text) > 3:
              text = text[:3];
            self.log_message(' / '.join(text),content=content,category=cat);
      # add to event log (if enabled)
      self.eventlog.add(str(ev),content=value,category=Logger.Event);
      # strip off index from end of event
      ev0 = ev;
      if int(ev0[-1]) >= 0:
        ev0 = ev0[:-1];
      # emit event as a PYSIGNAL so that registered handlers can get it
      self.emit(SIGNAL(str(ev).lower()),ev,value);
      ## NB: 31/01/2008: removed this, since we always send an attach event
      ## finally, just in case we've somehow missed a Hello message,
      ## force a connected call signal
      # if not self._connected and ev0 != hiid('bye'):
      #   self._attached_server_event(ev,value,server);
    except:
      (exctype,excvalue) = sys.exc_info()[:2];
      _dprint(0,'exception',str(exctype),'while handling event ',ev);
      traceback.print_exc();

  def _attached_server_event (self,ev,value,server):
    if not self._connected:
      self._connected = True;
      self.emit(SIGNAL("connected"),value,);
      self.emit(SIGNAL("isConnected"),True,);
      self.log_message("attached to server ("+str(server.addr)+")",category=Logger.Normal);
      self.gw.clear();

  def _detached_server_event (self,ev,value,server):
    if self._connected:
      self._connected = False;
      self.emit(SIGNAL("disconnected"),value,);
      self.emit(SIGNAL("isConnected"),False,);
      self.log_message("server detached",category=Logger.Normal);
    self._update_app_state();

##### updates status bar based on app state
  StatePixmaps = { None: pixmaps.red_round_cross };
  StatePixmap_Default = pixmaps.grey_cross;
  def _update_app_state (self):
    if self.app.current_server:
      state = self.app.current_server.statestr.lower();
      self.status_label.setText(' '+state+' ');
      _dprint(2,'app.state is',self.app.current_server.state,', setting pixmap');
      pm = self.StatePixmaps.get(self.app.current_server.state,self.StatePixmap_Default);
      self.status_icon.setPixmap(pm.pm());
      # update window title
      if self.app.current_server.addr is None:
        self.setWindowTitle(self.app.name()+" - "+state);
      else:
        self.setWindowTitle(str(self.app.current_server.addr)+" - "+state);
####### slot: pause button pressed
##  def _press_pause (self):
##    if self.pause_requested is None:
##      if self.app.paused:
##        self.pause_requested = False;
##        self.app.resume();
##      else:
##        self.pause_requested = True;
##        self.app.pause();
##    ## self.pause_button.setDown(True);
##### slot for the Event tab bar -- changes the label of a particular event logger
  def _change_eventlog_mask (self,logger,mask):
    self.eventtab.setTabText(self.eventtab.indexOf(logger),str(mask));
##### slot: adds error count to label of message logger
  def _indicate_msglog_errors (self,logger,numerr):
#    try: has_err = logger._numerr > 0;
#    except AttributeError: has_err = False;
#    # only add when going from or to 0 errors
#     if numerr and not has_err:
#       self.maintab.changeTab(logger,logger._error_icon,logger._error_label % numerr);
#     elif not numerr and has_err:
#       self._reset_maintab_label(logger);
#     logger._numerr = numerr;
    index = self.maintab.indexOf(logger);
    had_errors = getattr(self,'_had_errors',0);
    if numerr:
      if not had_errors:
        self.maintab.setTabIcon(index,logger._error_icon);
      self.maintab.setTabText(index,logger._error_label%numerr);
      self.maintab.show();
    else:
      self._reset_maintab_label(logger);
    self._had_errors = numerr;

  # resets tab label to default values
  def _reset_maintab_label (self,tabwin,icon=None,label=None):
    index = self.maintab.indexOf(tabwin);
    self.maintab.setTabIcon(index,icon or tabwin._default_icon);
    self.maintab.setTabText(index,label or tabwin._default_label);
    
  def log_message(self,msg,content=None,category=Logger.Normal):
    if category is not None and \
          self.msglog.add(msg,content=content,category=category,flood_protection=20,udi="message") and \
          self.maintab.currentWidget() is not self.msglog.wtop(): 
      self.statusbar.showMessage(msg,2000);

  def await_gui_exit ():
    global MainApp,MainAppThread;
    if MainAppThread:
      MainAppThread.join();
    else:
      try:
        MainApp.exec_();
      except KeyboardInterrupt: pass;
  await_gui_exit = staticmethod(await_gui_exit);

  class OverrideCursor (object):
    """When created, calls QApplication.setOverrideCursor() to change the app
    cursor. When destroyed, calls QApplication.restoreOverrideCursor() to
    restore the cursor. Handy to use as follows:
      tmp = OverrideCursor(Qt.WaitCursor);
      # do lenghty job
      # when 'tmp' goes out of scope, normal cursor automatically restored
    """
    def __init__ (self,cursor):
      QApplication.setOverrideCursor(QCursor(cursor));
    def __del__ (self):
      QApplication.restoreOverrideCursor();

  def override_cursor (cursor):
    """shortcut to create an override-cursor object""";
    return app_proxy_gui.OverrideCursor(cursor);
  override_cursor = staticmethod(override_cursor);

  def wait_cursor ():
    """shortcut to create an hourglass override-cursor""";
    return app_proxy_gui.OverrideCursor(Qt.WaitCursor);
  wait_cursor = staticmethod(wait_cursor);


MessageCategories = record( error   = Logger.Error,
                            message = Logger.Normal,
                            text    = Logger.Normal );

#--------------------------------------------------------------
#--- MainAppClass
#--------------------------------------------------------------
class MainAppClass (QApplication):
  _started = False;
  _waitcond = qt_threading.Condition();
  def __init__ (self,args):
    if self._started:
      raise "Only one MainApp may be started";
    QApplication.__init__(self,args);
    self.setDesktopSettingsAware(True);
#    # set 10pt font as default
#    font = self.font();
#    font.setPointSize(12);
#    self.setFont(font);
#    font = QFont("Georgia",10);
#    font.setStyleHint(QFont.System);
#    self.setFont(font);
    self.connect(self,SIGNAL("lastWindowClosed()"),self,SLOT("quit()"));
    ## NB: uncomment the line below to have Ctrl+C quit the app
    ## unconditionally (for alternative, see meqserver_gui.py)
    signal.signal(signal.SIGINT,self.__sigint_handler);
    # notify all waiters
    self._waitcond.acquire();
    self._started = True;
    self._waitcond.notifyAll();
    self._waitcond.release();

  def __sigint_handler (self,sig,stackframe):
    print('signal',sig);
    self.quit();

  def customEvent(self,ev):
    if isinstance(ev,QCallableEvent):
      ev.call();

  def postCallable(self,func,*args,**kwargs):
    self.postEvent(self,QCallableEvent(func,*args,**kwargs));


def appgui (widget):
  """finds app_proxy_gui parent of given widget, or None if none""";
  appgui = widget;
  while appgui and not isinstance(appgui,app_proxy_gui):
    appgui = appgui.parent();
  return appgui;

def mainapp ():
  """Creates if needed, and returns the MainApp object."""
  global MainApp;
  if not MainApp:
    MainApp = MainAppClass(sys.argv);
  return MainApp;

splash_screen = None;

def set_splash_screen (pm,message=None):
  mainapp();
  global splash_screen;
  splash_screen = QSplashScreen(pm());
  if message is not None:
    splash_screen.showMessage(message);
  splash_screen.show();

def mainapp_run ():
  MainApp.exec_();

def mainapp_threaded():
  """Creates the MainApp object and runs its event loop in its own thread.
  Not recommended."""
  global MainApp,MainAppThread;
  def _run_mainapp_thread ():
    global MainApp;
    # start the app
    MainApp = MainAppClass(sys.argv);
    # start the event loop thread
    MainApp.exec_();

  if MainAppClass._started:
    return MainApp;
  # start the main app in a separate thread
  MainAppThread = qt_threading.QThreadWrapper(_run_mainapp_thread);
  MainAppThread.start();
  # wait for start to complete
  MainAppClass._waitcond.acquire();
  while not MainAppClass._started:
    MainAppClass._waitcond.wait();
  MainAppClass._waitcond.release();
  return MainApp;

if __name__=="__main__":
  pass;



# -*- coding: utf-8 -*-
#/usr/bin/python
#
#% $Id: connect_meqtimba_dialog.py 6778 2009-02-19 14:00:37Z oms $ 
#
#
# Copyright (C) 2002-2007
# The MeqTree Foundation & 
# ASTRON (Netherlands Foundation for Research in Astronomy)
# P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>,
# or write to the Free Software Foundation, Inc., 
# 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

import Timba
from MeqGUI import Grid, GUI, Plugins
from MeqGUI.GUI import meqgui
from MeqGUI.GUI.pixmaps import pixmaps

from PyQt4.Qt import *
from Kittens.widgets import PYSIGNAL

try:
  from Timba.version_info.release import release
except:
  release = '(developer build)';
try:
  from Timba.version_info.svn_revision import svn_revision
  svn_revision = "<p align='right'>(svn revision %s)</p>"%svn_revision;
except:
  svn_revision = '';
  

class AboutDialog (QDialog):
    def __init__(self,parent=None,name=None,modal=0,fl=None):
        if fl is None:
          fl = Qt.Dialog|Qt.WindowTitleHint;
        QDialog.__init__(self,parent,Qt.Dialog|Qt.WindowTitleHint);
        self.setModal(modal);
        
        image0 = pixmaps.redhood_300.pm();

        # self.setSizeGripEnabled(0)
        LayoutWidget = QWidget(self)
        LayoutWidget.setSizePolicy(QSizePolicy.MinimumExpanding,QSizePolicy.MinimumExpanding);
        
        lo_top = QVBoxLayout(LayoutWidget)

        lo_title = QHBoxLayout(None)

        self.title_icon = QLabel(LayoutWidget)
        self.title_icon.setSizePolicy(QSizePolicy.Fixed,QSizePolicy.Fixed);
        self.title_icon.setPixmap(image0)
        self.title_icon.setAlignment(Qt.AlignCenter)
        lo_title.addWidget(self.title_icon)

        self.title_label = QLabel(LayoutWidget)
        self.title_label.setWordWrap(True);
        lo_title.addWidget(self.title_label)
        lo_top.addLayout(lo_title)
        
        if Timba.packages():
          lo_pkgs = QHBoxLayout(None)
          lo_top.addLayout(lo_pkgs);
          self.pkg_label = QLabel(LayoutWidget)
          self.pkg_label.setFrameStyle(QFrame.Box|QFrame.Raised);
          lo_pkgs.addWidget(self.pkg_label);
          txt = """<P>Optional packages:</P><TABLE>""";
          for pkg,(path,version) in Timba.packages().items():
            txt += """<TR><TD> %s </TD> <TD> at <tt>%s</tt> </TD> <TD> (%s) </TD></TR>"""%(pkg,path,version);
          txt += "</TABLE>";
          self.pkg_label.setText(txt);
        
        lo_logos = QHBoxLayout(None)
        lo_top.addLayout(lo_logos);
        for logo in "astron","oxford_physics","oerc","drao":
          icon = QLabel(LayoutWidget)
          icon.setSizePolicy(QSizePolicy.Fixed,QSizePolicy.Fixed);
          icon.setPixmap(getattr(pixmaps,logo+"_logo").pm());
          icon.setAlignment(Qt.AlignCenter)
          lo_logos.addWidget(icon)

        lo_mainbtn = QHBoxLayout(None)
        lo_mainbtn.addItem(QSpacerItem(20,20,QSizePolicy.Expanding,QSizePolicy.Minimum))
        lo_top.addLayout(lo_mainbtn);

        self.btn_ok = QPushButton(LayoutWidget)
        self.btn_ok.setSizePolicy(QSizePolicy.Fixed,QSizePolicy.Fixed);
        self.btn_ok.setMinimumSize(QSize(60,0))
        self.btn_ok.setAutoDefault(1)
        self.btn_ok.setDefault(1)
        lo_mainbtn.addWidget(self.btn_ok)
        lo_mainbtn.addItem(QSpacerItem(20,20,QSizePolicy.Expanding,QSizePolicy.Minimum))

        self.languageChange()
        
        LayoutWidget.adjustSize();

        #LayoutWidget.resize(QSize(489,330).expandedTo(LayoutWidget.minimumSizeHint()))
        #self.resize(QSize(489,330).expandedTo(self.minimumSizeHint()))
        # self.clearWState(Qt.WState_Polished)
        
        self.connect(self.btn_ok,SIGNAL("clicked()"),self.accept)
        
    def languageChange(self):
        self.setWindowTitle(self.__tr("About MeqTrees"))
        self.title_label.setText(self.__tr( \
          """<h1>MeqTrees %s</h1>
          %s<p>(C) 2002-2009 ASTRON<br>(Netherlands Institude for Radioastronomy)<br>
          Oude Hoogeveensedijk 4<br>
          7991 PD Dwingeloo, The Netherlands<br>
          http://www.astron.nl</p>
          <p>With contributions from:<p><ul>
          <li>Oxford Astrophysics<li>
          <li>Oxford e-Research Centre<li>
          <li>Dominion Radio Astrophysical Observatory (NRC-HIA)</li>
          <li>Sarod Yatawatta, Tony Willis, Maaijke Mevius,
          Ger van Diepen, Ronald Nijboer, Filipe Abdalla, Rob Assendorp, Ilse
          van Bemmel, Ian Heywood, Hans-Rainer Kloeckner, Rense Boomsma, Michiel
          Brentjens, Joris van Zwieten, Stef Salvini, Christopher Williams, Mike Sipior,
          James Anderson, George Heald,  Jan Noordam, Oleg Smirnov
          </ul>
          """%(release,svn_revision) \
          ));

        self.btn_ok.setText(self.__tr("&OK"))


    def __tr(self,s,c = None):
        return qApp.translate("ConnectMeqKernel",s,c)

